<?php

declare(strict_types=1);

namespace FiloBlu\Refilo\Helper\Catalog;

use FiloBlu\Refilo\Remote\Entity\Product;
use FiloBlu\Refilo\Remote\Entity\Provider\PriceProvider;
use FiloBlu\Refilo\Remote\Entity\Provider\StockProvider;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class ProductHelper
 * @package FiloBlu\Refilo\Helper\Catalog
 */
class ProductHelper
{

    /**
     * @var array
     */
    protected $prices = null;

    /**
     * @var array
     */
    protected $stocks = null;

    /**
     * @var StockProvider
     */
    private $stockProvider;

    /**
     * @var PriceProvider
     */
    private $priceProvider;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * ProductHelper constructor.
     * @param StoreManagerInterface $storeManager
     * @param StockProvider $stockProvider
     * @param PriceProvider $priceProvider
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        StockProvider $stockProvider,
        PriceProvider $priceProvider
    ) {
        $this->stockProvider = $stockProvider;
        $this->priceProvider = $priceProvider;
        $this->storeManager = $storeManager;
    }

    /**
     * @param ProductInterface $product
     * @return mixed
     * @throws NoSuchEntityException
     */
    public function getStockFor(ProductInterface $product)
    {
        $storeId = (int)$product->getStoreId();
        $entityId = $product->getEntityId();

        if ($this->stocks === null || !isset($this->stocks[$storeId])) {
            $store = $this->storeManager->getStore($storeId);
            foreach (iterator_to_array($this->stockProvider->forStore($store)) as $stock) {
                $this->stocks[$storeId][$stock->getId()] = $stock->getData(Product::INVENTORY);
            }
        }

        if (isset($this->stocks[$storeId][$entityId])) {
            return $this->stocks[$storeId][$entityId];
        }

        return [];
    }

    /**
     * @param ProductInterface $product
     * @return mixed
     * @throws NoSuchEntityException
     */
    public function getPricesFor(ProductInterface $product)
    {
        $storeId = (int)$product->getStoreId();
        $entityId = (int)$product->getEntityId();

        if ($this->prices === null || !isset($this->prices[$storeId])) {
            $store = $this->storeManager->getStore($storeId);
            foreach (iterator_to_array($this->priceProvider->forStore($store)) as $price) {
                $this->prices[$storeId][$price->getId()] = $price->getData();
            }
        }

        if (isset($this->prices[$storeId][$entityId])) {
            return $this->prices[$storeId][$entityId];
        }

        return [];
    }
}
